#!/usr/bin/env python

"""Tests for `zoslogs` class."""

import datetime

import pytest

from src.zoslogs import utils


def test_five_digits():
    input_date = '05132'

    correct_ouput = datetime.datetime(2005, 5, 12)

    assert utils.julian_to_datetime(input_date) == correct_ouput


def test_seven_digits():
    input_date = '2012313'

    correct_output = datetime.datetime(2012, 11, 8)

    assert utils.julian_to_datetime(input_date) == correct_output


def test_six_digits():
    inputdate = '012313'

    with pytest.raises(utils.UtilityException):
        utils.julian_to_datetime(inputdate)


def test_invalid_date():
    inputdate = '1943742'

    with pytest.raises(ValueError):
        utils.julian_to_datetime(inputdate)



#!/usr/bin/env python

"""Tests for `zoslogs` zosmessage class."""

import pytest

from src.zoslogs import zosmessage


def test_single_line_message():
    with open('tests/test_data/single_line_message.txt') as file:
        line = file.readline()

        message = zosmessage.ZosMessage(line)

    assert message.record_type == 'N'
    assert message.request_type == ' '
    assert message.routing_codes == '4000000'
    assert message.sysname == 'S5A'
    assert message.julian_date == '22039'
    assert message.time == '00:01:00.55'
    assert message.user_exit_mpf_flags == '00000211'

    assert message.message == '$HASP373 LOGRSVT  STARTED - INIT 15   - CLASS Q        - SYS S5A\n'
    assert message.message_id == '$HASP373'
    # assert message.jobname == 'J0126429'


def test_single_line_command():
    with open('tests/test_data/single_line_command.txt') as file:
        line = file.readline()

        message = zosmessage.ZosMessage(line)

    assert message.record_type == 'N'
    assert message.request_type == 'C'
    assert message.routing_codes == '0000000'
    assert message.sysname == 'S5A'
    assert message.julian_date == '22039'
    assert message.time == '07:00:34.88'
    assert message.user_exit_mpf_flags == '00000210'
    assert message.console_id == 'INTERNAL'

    assert message.message == 'CANCEL   LOGRSVTS,A=0064\n'
    # assert message.jobname == 'J0126429'


def test_single_line_internal():
    with open('tests/test_data/single_line_internal.txt') as file:
        line = file.readline()

        message = zosmessage.ZosMessage(line)

    assert message.record_type == 'N'
    assert message.request_type == 'I'
    assert message.routing_codes == '0000000'
    assert message.sysname == 'S5A'
    assert message.julian_date == '22039'
    assert message.time == '07:00:34.88'
    assert message.user_exit_mpf_flags == '00000210'
    assert message.console_id is None
    assert message.job_id == 'J0130811'

    assert message.message == '$CJ0130811,PURGE\n'
    # assert message.jobname == 'J0126429'


def test_syslog_initilization():
    with open('tests/test_data/syslog_initialization_message.txt') as file:
        line = file.readline()

        message = zosmessage.ZosMessage(line)

    assert message.record_type == 'X'
    assert message.routing_codes == '0000000'
    assert message.sysname == 'S5A'
    assert message.julian_date == '21310'
    assert message.time == '00:00:01.81'
    assert message.user_exit_mpf_flags == '00000000'
    assert message.console_id is None
    assert message.job_id == 'SYSLOG'
    assert message.message_id == 'IEE042I'

    assert message.message == 'IEE042I SYSTEM LOG DATA SET INITIALIZED\n'


def test_multi_line_message():
    with open('tests/test_data/multi_line.txt') as file:
        lines = file.readlines()
        message = zosmessage.ZosMessage(lines)

    assert message.record_type == 'M'
    assert message.request_type == ' '
    assert message.routing_codes == '0040000'
    assert message.sysname == 'S5A'
    assert message.julian_date == '22039'
    assert message.time == '03:54:38.76'
    assert message.user_exit_mpf_flags == '00000210'
    assert message.job_id == 'J0120104'
    assert message.message_id == "IXL030I"

    assert message.message == 'IXL030I CONNECTOR STATISTICS FOR LOCK STRUCTURE COACMELCKIMS_1,\n' \
                              'CONNECTOR COACMELC0B000019:\n' \
                              '      00071011 00000000 00000062 00F0000F 000A\n' \
                              '\n' \
                              '      00000000 00000000 00000001 00000000\n' \
                              '      00000000 00000000 00000000 00000005\n' \
                              '      00000000 00000000 00000000 00000000\n' \
                              '      00000000 00000000 00000000 00000000\n' \
                              '\n' \
                              '      00000001 00000000 00000001 00000000\n' \
                              '      00000000 00000000 00000000 0000009D\n' \
                              '      00000000 00000000 00000000 00000000\n' \
                              '      00000000 00000000 00000000 00000000\n' \
                              '\n' \
                              '      00000002 00000000 00000001 00000000\n' \
                              '      00000000 00000000 00000000 00000065\n' \
                              '      00000000 00000000 00000000 00000000\n' \
                              '      00000000 00000000 00000000 00000000\n' \
                              '\n' \
                              '      00000003 00000000 00000000 00000000\n' \
                              '      00000000 00000000 00000000 00000000\n' \
                              '      00000000 00000000 00000000 00000000\n' \
                              '      00000000 00000000 00000000 00000000\n'


def test_empty_line():
    with open('tests/test_data/empty_line.txt') as file:
        line = file.readline()
        with pytest.raises(zosmessage.MessageException):
            zosmessage.ZosMessage(line)


def test_empty_multi_line():
    with open('tests/test_data/empty_multi_line.txt') as file:
        line = file.readline()
        with pytest.raises(zosmessage.MessageException):
            zosmessage.ZosMessage(line)


def test_single_line_messageid_filter_match():
    with open('tests/test_data/single_line_message.txt') as file:
        line = file.readline()

        filter = {"message_ids_to_include": ['$HASP373']}

        message = zosmessage.ZosMessage(line, message_filters=filter)

    assert message.record_type == 'N'
    assert message.request_type == ' '
    assert message.routing_codes == '4000000'
    assert message.sysname == 'S5A'
    assert message.julian_date == '22039'
    assert message.time == '00:01:00.55'
    assert message.user_exit_mpf_flags == '00000211'

    assert message.message == '$HASP373 LOGRSVT  STARTED - INIT 15   - CLASS Q        - SYS S5A\n'
    assert message.message_id == '$HASP373'


def test_single_line_message_contains_filter_match():
    with open('tests/test_data/single_line_message.txt') as file:
        line = file.readline()

        filter = {"message_contains": ['INIT 15']}

        message = zosmessage.ZosMessage(line, message_filters=filter)

    assert message.record_type == 'N'
    assert message.request_type == ' '
    assert message.routing_codes == '4000000'
    assert message.sysname == 'S5A'
    assert message.julian_date == '22039'
    assert message.time == '00:01:00.55'
    assert message.user_exit_mpf_flags == '00000211'

    assert message.message == '$HASP373 LOGRSVT  STARTED - INIT 15   - CLASS Q        - SYS S5A\n'
    assert message.message_id == '$HASP373'


def test_single_line_message_contains_message_id_and_message_filter_match():
    with open('tests/test_data/single_line_message.txt') as file:
        line = file.readline()

        filter = {"message_contains": ['INIT 15'],
                  "message_ids_to_include": ['$HASP373']}

        message = zosmessage.ZosMessage(line, message_filters=filter)

    assert message.record_type == 'N'
    assert message.request_type == ' '
    assert message.routing_codes == '4000000'
    assert message.sysname == 'S5A'
    assert message.julian_date == '22039'
    assert message.time == '00:01:00.55'
    assert message.user_exit_mpf_flags == '00000211'

    assert message.message == '$HASP373 LOGRSVT  STARTED - INIT 15   - CLASS Q        - SYS S5A\n'
    assert message.message_id == '$HASP373'


def test_single_line_messageid_filter_no_match():
    with open('tests/test_data/single_line_message.txt') as file:
        line = file.readline()

        filter = {"message_ids_to_include": ['$HASP374']}

        with pytest.raises(zosmessage.FilterException):
            zosmessage.ZosMessage(line, message_filters=filter)


def test_single_line_message_contains_filter_no_match():
    with open('tests/test_data/single_line_message.txt') as file:
        line = file.readline()

        filter = {"message_contains": ['INIT 16']}

        with pytest.raises(zosmessage.FilterException):
            zosmessage.ZosMessage(line, message_filters=filter)


def test_multi_line_message_no_messagenum():
    with open('tests/test_data/multi_line_no_messagenum.txt') as file:
        lines = file.readlines()
        with pytest.raises(zosmessage.MessageException):
            zosmessage.ZosMessage(lines)


def test_str():
    with open('tests/test_data/single_line_internal.txt') as file:
        line = file.readline()

        message = zosmessage.ZosMessage(line)

        assert str(message) == "22039 07:00:34.88 S5A      $CJ0130811,PURGE"



import datetime


def julian_to_datetime(input_string: str):
    """

    :param: input_string String to be converted
    :rtype: datetime object
    """
    if len(input_string) == 5:

        date = datetime.datetime.strptime(input_string, '%y%j')

    elif len(input_string) == 7:

        date = datetime.datetime.strptime(input_string, '%Y%j')
    else:
        raise UtilityException("Incorrect parameter length passed to "
                               "julian_to_datetime")

    return date


class UtilityException(Exception):
    pass



"""Main module."""

import logging

from tqdm import tqdm

from src.zoslogs.zosmessage import ZosMessage, MessageException, \
    FilterException


class ZosLogs():

    def __init__(self, text_stream, message_filters=None, halt_on_errors=False, disable_tqdm=None):
        """

        :param text_stream:  A text stream to break into messages
        :param message_filters:  A dictionary of filters
        :param halt_on_errors:  By default, the library will do its best to
                                parse the logs, and ignore anything it can't
                                understand.  If this is True, the library will
                                raise an exception if it has problems parsing
                                something, instead parsing what it does
                                understand
        :param disable_tqdm:  By default, the library will use tqdm to display
                              parsing progress when running interactively.  If
                              this is set to True, it won't display parsing
                              progress.
        """

        logger = logging.getLogger(__name__)

        self.message_list = list()

        messages = dict()

        # Going to loop through twice for now, to reduce complexity.
        # Thinking of a better way to handle this.

        cleaned_text_stream = list()

        logger.info("Cleaning text")

        for line in tqdm(text_stream, disable=disable_tqdm, desc="Cleaning text", leave=False):

            # Syslog starts with a leading space; Operlog doesn't.  So, we'll drop leading spaces.

            logger.debug(line)

            # Try to discard any characters I don't know what to do with
            # (Problem if using an older version of operlog dump program)
            line = line.encode("ascii", errors="ignore").decode().lstrip()

            if len(line) == 0:
                continue

            # Can happen when a virtual page is created by syslog
            if line[0].isdigit():
                line = line[1:]

            # Next page
            if line[0] == "+":
                continue

            # Continuation of prior line
            if line[0] == "S":
                try:
                    cleaned_text_stream[-1] = cleaned_text_stream[-1].rstrip() + " " + \
                                              line[1:].lstrip()
                except IndexError:
                    error = "Trying to add message to prior message, but there is no prior " \
                            "message\n" + line
                    if halt_on_errors:
                        raise MessageException(error)
                    else:
                        logger.error(error)

                continue

            cleaned_text_stream.append(line)

        logger.info("Processing cleaned lines")

        for line in tqdm(cleaned_text_stream, disable=disable_tqdm,
                         desc="Processing messages", leave=False):

            new_message = None

            # Single Line message or Syslog initialization message
            if line[0] == "N" or line[0] == "X":

                try:
                    new_message = ZosMessage(line, message_filters=message_filters)
                except FilterException:
                    continue
                except MessageException as e:
                    if halt_on_errors:
                        raise e
                    else:
                        logger.exception(e)

            # Multiline message Start
            elif line[0] == "M":

                multiline_id = line.split()[-1]

                try:
                    assert multiline_id.isnumeric()
                except AssertionError as e:
                    logger.warning("Got multiline id of " + str(multiline_id))
                    logger.warning(line)
                    if halt_on_errors:
                        raise e

                messages[multiline_id] = [line]

            # Multiline data or list
            elif line[0] == "D" or line[0] == "L":

                multiline_id = line[42:45]

                try:
                    messages[multiline_id].append(line)
                except KeyError as e:
                    error_message = ("Trying to append data to multiline message " + multiline_id +
                                     " with no such message header")
                    logger.warning(error_message)
                    if halt_on_errors:
                        raise MessageException(error_message) from e

            # Multiline end
            elif line[0] == "E":

                multiline_id = line[42:45]

                try:
                    messages[multiline_id].append(line)
                except KeyError as e:
                    logger.warning("Trying to append data to multiline message " + multiline_id +
                                   " with no such message header")
                    if halt_on_errors:
                        raise e

                try:
                    new_message = ZosMessage(messages[multiline_id],
                                             message_filters=message_filters)
                    messages.pop(multiline_id)
                except FilterException:
                    continue
                except MessageException as e:
                    logger.warning("Error parsing " + str(messages[multiline_id]))
                    if halt_on_errors:
                        raise e
                except KeyError as e:
                    logger.warning("Got a multiline ending " + multiline_id +
                                   " with no header")
                    if halt_on_errors:
                        raise e

            else:
                logger.warning("I don't know what to do with message record "
                               "type " + line[0] + "\n" + line)
                logger.warning("Skipping")

            if new_message:
                self.message_list.append(new_message)

        if len(messages) > 0:
            logger.warning("Have multiline messages I never saw an ending for")

        for message_id in messages:

            logger.warning(messages[message_id])

            try:
                new_message = ZosMessage(messages[message_id], message_filters=message_filters)
            except FilterException:
                continue
            except MessageException as e:
                logger.warning("Error parsing " + str(messages[message_id]))
                if halt_on_errors:
                    raise e
                else:
                    continue

            if new_message is not None:
                self.message_list.append(new_message)

    def __len__(self):
        return len(self.message_list)

    def __yield__(self):
        return self

    def __next__(self):
        return (self.message_list.pop())

    def __getitem__(self, item):
        return (self.message_list[item])



import logging
import datetime


class ZosMessage():

    def __init__(self, text_to_parse: list, message_filters: dict = None):
        """

        :param text_to_parse: a list of one or more lines to parse
        :param message_filters:  messages to filter on; if filters are
        specified, and the message doesn't meet the filter requirements,
        raise a MessageFilter exception
        :param loglevel: the log level
        """

        log = logging.getLogger(__name__)

        self.record_type: str = None
        self.request_type: str = None
        self.routing_codes: str = None
        self.julian_date: int = None
        self.time: str = None
        self.date_time: datetime = None
        self.sysname: str = None
        self.console_id: str = None
        self.job_id: str = None
        self.user_exit_mpf_flags: str = None

        self.message: str = None
        self.message_id: str = None

        if isinstance(text_to_parse, str):
            first_line = text_to_parse
            rest_of_message = None
        elif isinstance(text_to_parse, list):
            first_line = text_to_parse[0]
            rest_of_message = text_to_parse[1:]

        self.record_type = first_line[0]
        self.request_type = first_line[1]

        self.routing_codes = first_line[2:9]
        self.sysname = first_line[10:18].rstrip()

        self.julian_date = first_line[19:24]
        self.time = first_line[25:36]

        self.user_exit_mpf_flags = first_line[46:54]

        self.message = first_line[56:]

        # Single Line Command Issued by operator
        if self.record_type == "N" and self.request_type == "C":
            self.console_id = first_line[37:45].rstrip()

        # Single Line Internal Message
        if self.record_type == "N" and (self.request_type == "I" or self.request_type == ' ')\
           or self.record_type == "X":
            self.job_id = first_line[37:45].rstrip()

            try:
                self.message_id = self.message.split()[0]
            except IndexError as e:
                log.error("Problem determining message id for line " + text_to_parse)
                raise MessageException("Problem determining message id for line " +
                                       text_to_parse) from e

        if self.record_type == "M" and (self.request_type == " "):
            self.job_id = first_line[37:45]
            try:
                self.message_id = self.message.split()[0]
            except IndexError as e:
                log.error("Problem determining message id for line " + str(text_to_parse))
                raise MessageException("Problem determining message id for line " +
                                       str(text_to_parse)) from e

            message_num = first_line.split()[-1]

            try:
                assert message_num.isnumeric()
            except AssertionError as e:
                raise MessageException("Error parsing " + str(text_to_parse) +
                                       "Missing message number") from e

            self.message = self.message.rpartition(message_num)[0].rstrip() + "\n"

        if message_filters is not None:
            if 'message_ids_to_include' in message_filters:
                if self.message_id not in message_filters['message_ids_to_include']:
                    raise FilterException

        if rest_of_message:
            for line in rest_of_message:

                if len(line) > 56:
                    self.message += line[56:]
                else:
                    self.message += '\n'

        if message_filters is not None:
            if 'message_contains' in message_filters:
                if any(filter_item in self.message for filter_item in
                       message_filters["message_contains"]):
                    pass
                else:
                    raise FilterException

    def __str__(self):
        return ' '.join((self.julian_date, self.time, self.sysname.ljust(8),
                         self.message)).rstrip()


class FilterException(Exception):
    pass


class MessageException(Exception):
    pass



#!/usr/bin/env python

"""The setup script."""

import io
import re
from glob import glob
from os.path import basename
from os.path import dirname
from os.path import join
from os.path import splitext

from setuptools import find_packages
from setuptools import setup


def read(*names, **kwargs):
    with io.open(
        join(dirname(__file__), *names),
        encoding=kwargs.get('encoding', 'utf8')
    ) as fh:
        return fh.read()

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [ ]

test_requirements = ['pytest>=3', ]

setup(
    author="Kevin McKenzie",
    author_email='kmckenzi@us.ibm.com',
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
    description="Library for breaking z/OS log files up into individual messages",
    entry_points={
        'console_scripts': [
            'zoslogs=zoslogs.zoslogscli:main',
        ],
    },
    install_requires=requirements,
    license="Apache Software License 2.0",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='zoslogs',
    name='zoslogs',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    py_modules=[splitext(basename(path))[0] for path in glob('src/*.py')],
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/kmckenzi/zoslogs',
    version='0.1.1',
    zip_safe=False,
)



#!/usr/bin/env python

"""Tests for `zoslogs` class."""

import fileinput
import pytest

from src.zoslogs import zoslogs
from src.zoslogs.zosmessage import MessageException


def test_single_line_message():
    with open('tests/test_data/single_line_message.txt') as file:

        log = zoslogs.ZosLogs(file)

    assert len(log) == 1

    assert log[0].record_type == 'N'
    assert log[0].request_type == ' '
    assert log[0].routing_codes == '4000000'
    assert log[0].sysname == 'S5A'
    assert log[0].julian_date == '22039'
    assert log[0].time == '00:01:00.55'
    assert log[0].user_exit_mpf_flags == '00000211'

    assert log[0].message == '$HASP373 LOGRSVT  STARTED - INIT 15   - CLASS Q        - SYS S5A\n'
    # assert log[0].message_id == '$HASP373'
    # assert log[0].jobid == 'J0126429'


def test_single_line_command():
    with open('tests/test_data/single_line_command.txt') as file:
        log = zoslogs.ZosLogs(file)

    assert log[0].record_type == 'N'
    assert log[0].request_type == 'C'
    assert log[0].routing_codes == '0000000'
    assert log[0].sysname == 'S5A'
    assert log[0].julian_date == '22039'
    assert log[0].time == '07:00:34.88'
    assert log[0].user_exit_mpf_flags == '00000210'
    assert log[0].console_id == 'INTERNAL'

    assert log[0].message == 'CANCEL   LOGRSVTS,A=0064\n'
    # assert log[0].jobname == 'J0126429'


def test_single_line_internal():
    with open('tests/test_data/single_line_internal.txt') as file:
        log = zoslogs.ZosLogs(file)

    assert log[0].record_type == 'N'
    assert log[0].request_type == 'I'
    assert log[0].routing_codes == '0000000'
    assert log[0].sysname == 'S5A'
    assert log[0].julian_date == '22039'
    assert log[0].time == '07:00:34.88'
    assert log[0].user_exit_mpf_flags == '00000210'
    assert log[0].console_id is None
    assert log[0].job_id == 'J0130811'

    assert log[0].message == '$CJ0130811,PURGE\n'
    # assert log[0].jobname == 'J0126429'


def test_message_with_continuation():
    with open('tests/test_data/single_line_with_continuation.txt') as file:

        logs = zoslogs.ZosLogs(file)

    assert logs[0].record_type == 'N'
    assert logs[0].request_type == ' '
    assert logs[0].routing_codes == '0004000'
    assert logs[0].sysname == 'S5A'
    assert logs[0].julian_date == '22039'
    assert logs[0].time == '07:00:06.11'
    assert logs[0].user_exit_mpf_flags == '00000210'

    assert logs[0].message == '-LOGRSVTS ENDED.  NAME-                     ' \
                              'TOTAL CPU TIME=   .00  TOTAL ELAPSED TIME=    .0\n'
    # assert log[0].jobname == 'J0126429'


def test_multi_line_message():
    with open('tests/test_data/multi_line.txt') as file:
        logs = zoslogs.ZosLogs(file)

    assert logs[0].record_type == 'M'
    assert logs[0].request_type == ' '
    assert logs[0].routing_codes == '0040000'
    assert logs[0].sysname == 'S5A'
    assert logs[0].julian_date == '22039'
    assert logs[0].time == '03:54:38.76'
    assert logs[0].user_exit_mpf_flags == '00000210'
    assert logs[0].job_id == 'J0120104'

    assert logs[0].message == 'IXL030I CONNECTOR STATISTICS FOR LOCK STRUCTURE COACMELCKIMS_1,\n' \
                              'CONNECTOR COACMELC0B000019:\n' \
                              '      00071011 00000000 00000062 00F0000F 000A\n' \
                              '\n' \
                              '      00000000 00000000 00000001 00000000\n' \
                              '      00000000 00000000 00000000 00000005\n' \
                              '      00000000 00000000 00000000 00000000\n' \
                              '      00000000 00000000 00000000 00000000\n' \
                              '\n' \
                              '      00000001 00000000 00000001 00000000\n' \
                              '      00000000 00000000 00000000 0000009D\n' \
                              '      00000000 00000000 00000000 00000000\n' \
                              '      00000000 00000000 00000000 00000000\n' \
                              '\n' \
                              '      00000002 00000000 00000001 00000000\n' \
                              '      00000000 00000000 00000000 00000065\n' \
                              '      00000000 00000000 00000000 00000000\n' \
                              '      00000000 00000000 00000000 00000000\n' \
                              '\n' \
                              '      00000003 00000000 00000000 00000000\n' \
                              '      00000000 00000000 00000000 00000000\n' \
                              '      00000000 00000000 00000000 00000000\n' \
                              '      00000000 00000000 00000000 00000000\n'


def test_multi_line_message_with_missing_message_header():
    with open('tests/test_data/multi_line_with_no_message_header.txt') as file:
        logs = zoslogs.ZosLogs(file)

    assert len(logs) == 1

    assert logs[0].record_type == 'M'
    assert logs[0].request_type == ' '
    assert logs[0].routing_codes == '0040000'
    assert logs[0].sysname == 'S5A'
    assert logs[0].julian_date == '22039'
    assert logs[0].time == '03:54:38.76'
    assert logs[0].user_exit_mpf_flags == '00000210'
    assert logs[0].job_id == 'J0120104'

    assert logs[0].message == 'IXL030I CONNECTOR STATISTICS FOR LOCK STRUCTURE COACMELCKIMS_1,\n' \
                              'CONNECTOR COACMELC0B000019:\n' \
                              '      00071011 00000000 00000062 00F0000F 000A\n' \
                              '\n' \
                              '      00000000 00000000 00000001 00000000\n' \
                              '      00000000 00000000 00000000 00000005\n' \
                              '      00000000 00000000 00000000 00000000\n' \
                              '      00000000 00000000 00000000 00000000\n' \
                              '\n' \
                              '      00000001 00000000 00000001 00000000\n' \
                              '      00000000 00000000 00000000 0000009D\n' \
                              '      00000000 00000000 00000000 00000000\n' \
                              '      00000000 00000000 00000000 00000000\n' \
                              '\n' \
                              '      00000002 00000000 00000001 00000000\n' \
                              '      00000000 00000000 00000000 00000065\n' \
                              '      00000000 00000000 00000000 00000000\n' \
                              '      00000000 00000000 00000000 00000000\n' \
                              '\n' \
                              '      00000003 00000000 00000000 00000000\n' \
                              '      00000000 00000000 00000000 00000000\n' \
                              '      00000000 00000000 00000000 00000000\n' \
                              '      00000000 00000000 00000000 00000000\n'


def test_multi_line_message_with_missing_message_header_and_halt():
    with open('tests/test_data/multi_line_with_no_message_header.txt') as file:
        with pytest.raises(MessageException):
            zoslogs.ZosLogs(file, halt_on_errors=True)


def test_multi_line_message_with_continuation():
    with open('tests/test_data/multi_line_with_continuation.txt') as file:
        logs = zoslogs.ZosLogs(file)

    assert logs[0].record_type == 'M'
    assert logs[0].request_type == ' '
    assert logs[0].routing_codes == '0000000'
    assert logs[0].sysname == 'S5A'
    assert logs[0].julian_date == '22039'
    assert logs[0].time == '09:03:08.10'
    assert logs[0].user_exit_mpf_flags == '00000210'
    assert logs[0].job_id == 'S0119482'


def test_multiple_messages():
    with fileinput.input(('tests/test_data/multi_line.txt',
                          'tests/test_data/single_line_with_continuation.txt',
                          'tests/test_data/single_line_internal.txt',
                          'tests/test_data/single_line_command.txt',
                          'tests/test_data/multi_line_with_continuation.txt')) \
                          as files:
        log = zoslogs.ZosLogs(files)

    assert len(log) == 5


def test_new_syslog_page():
    with fileinput.input(('tests/test_data/new_syslog_page.txt',
                          'tests/test_data/syslog_initialization_message.txt')) \
                        as files:
        log = zoslogs.ZosLogs(files)

    assert len(log) == 2


def test_multiple_messages_with_filters():
    filter = {"message_ids_to_include": ['IEE043I']}

    with fileinput.input(('tests/test_data/multiple_messages.txt')) as file:
        log = zoslogs.ZosLogs(file, message_filters=filter)

    assert len(log) == 1


def test_multiple_messages_with_filters_and_exception():
    filter = {"message_ids_to_include": ['IEE043I']}

    with fileinput.input(('tests/test_data/multiple_messages.txt',
                          'tests/test_data/multi_line_with_no_message_header.txt')) as file:
        log = zoslogs.ZosLogs(file, message_filters=filter)

    assert len(log) == 1


def test_multiple_messages_with_filters_and_exception_and_halt():
    filter = {"message_ids_to_include": ['IEE043I']}

    with fileinput.input(('tests/test_data/multiple_messages.txt',
                          'tests/test_data/multi_line_with_no_message_header.txt')) as file:
        with pytest.raises(MessageException):
            zoslogs.ZosLogs(file, message_filters=filter, halt_on_errors=True)
